@@ -157,9 +157,13 @@ struct sheepdog_obj_req {
 	uint32_t data_length;
 	uint64_t oid;
 	uint64_t cow_oid;
-	uint32_t copies;
+	uint8_t copies;
+	uint8_t copy_policy;
+	uint8_t ec_index;
+	uint8_t reserved;
 	uint32_t rsvd;
-	uint64_t offset;
+	uint32_t offset;
+	uint32_t pad;
 };
 
 struct sheepdog_obj_rsp {
@@ -170,7 +174,8 @@ struct sheepdog_obj_rsp {
 	uint32_t id;
 	uint32_t data_length;
 	uint32_t result;
-	uint32_t copies;
+	uint8_t copies;
+	uint8_t reserved[3];
 	uint32_t pad[6];
 };
 
@@ -186,7 +191,10 @@ struct sheepdog_vdi_req {
 	uint32_t data_length;
 	uint64_t vdi_size;
 	uint32_t vdi_id;
-	uint32_t copies;
+	uint8_t copies;
+	uint8_t copy_policy;
+	uint8_t ec_index;
+	uint8_t block_size_shift;
 	uint32_t snapid;
 	uint32_t type;
 	uint32_t pad[2];
@@ -202,7 +210,11 @@ struct sheepdog_vdi_rsp {
 	uint32_t result;
 	uint32_t rsvd;
 	uint32_t vdi_id;
-	uint32_t pad[5];
+	uint32_t attr_id;
+	uint8_t copies;
+	uint8_t block_size_shift;
+	uint8_t reserved[2];
+	uint32_t pad[3];
 };
 
 /*
@@ -639,7 +651,7 @@ static int find_vdi_name(struct sheepdog_access_info *ai, char *filename,
 			 uint32_t snapid, char *tag, uint32_t *vid,
 			 int for_snapshot);
 static int read_object(struct sheepdog_access_info *ai, char *buf, uint64_t oid,
-		       int copies, unsigned int datalen, uint64_t offset,
+		       int copies, unsigned int datalen, uint32_t offset,
 		       int *need_reload);
 
 static int reload_inode(struct sheepdog_access_info *ai, int is_snapshot)
@@ -679,7 +691,7 @@ static int reload_inode(struct sheepdog_access_info *ai, int is_snapshot)
 
 static int read_write_object(struct sheepdog_access_info *ai, char *buf,
 			     uint64_t oid, int copies,
-			     unsigned int datalen, uint64_t offset,
+			     unsigned int datalen, uint32_t offset,
 			     int write, int create, uint64_t old_oid,
 			     uint16_t flags, int *need_reload)
 {
@@ -738,7 +750,7 @@ static int read_write_object(struct sheepdog_access_info *ai, char *buf,
 
 static int read_object(struct sheepdog_access_info *ai, char *buf,
 		       uint64_t oid, int copies,
-		       unsigned int datalen, uint64_t offset, int *need_reload)
+		       unsigned int datalen, uint32_t offset, int *need_reload)
 {
 	return read_write_object(ai, buf, oid, copies, datalen, offset,
 				 0, 0, 0, 0, need_reload);
@@ -746,7 +758,7 @@ static int read_object(struct sheepdog_access_info *ai, char *buf,
 
 static int write_object(struct sheepdog_access_info *ai, char *buf,
 			uint64_t oid, int copies,
-			unsigned int datalen, uint64_t offset, int create,
+			unsigned int datalen, uint32_t offset, int create,
 			uint64_t old_oid, uint16_t flags, int *need_reload)
 {
 	return read_write_object(ai, buf, oid, copies, datalen, offset, 1,
@@ -853,12 +865,13 @@ static int sd_io(struct sheepdog_access_info *ai, int write, char *buf, int len,
 		 uint64_t offset)
 {
 	uint32_t vid;
-	unsigned long idx = offset / SD_DATA_OBJ_SIZE;
+	uint32_t object_size = (UINT32_C(1) << ai->inode.block_size_shift);
+	unsigned long idx = offset / object_size;
 	unsigned long max =
-		(offset + len + (SD_DATA_OBJ_SIZE - 1)) / SD_DATA_OBJ_SIZE;
-	unsigned obj_offset = offset % SD_DATA_OBJ_SIZE;
+		(offset + len + (object_size - 1)) / object_size;
+	unsigned obj_offset = offset % object_size;
 	size_t orig_size, size, rest = len;
-	int ret = 0, create = 0;
+	int ret = 0, create;
 	uint64_t oid, old_oid;
 	uint16_t flags = 0;
 	int need_update_inode = 0, need_reload_inode;
@@ -885,7 +898,7 @@ do_req:
 
 	for (; idx < max; idx++) {
 		orig_size = size;
-		size = SD_DATA_OBJ_SIZE - obj_offset;
+		size = object_size - obj_offset;
 		size = min_t(size_t, size, rest);
 
 retry:
@@ -899,6 +912,7 @@ retry:
 			 * always turn on cache of sheep layer
 			 */
 			flags = SD_FLAG_CMD_CACHE;
+			create = 0;
 
 			if (ai->inode.data_vdi_id[idx] != vid) {
 				create = 1;
@@ -1196,7 +1210,6 @@ trans_to_expect_nothing:
 	ai->min_dirty_data_idx = UINT32_MAX;
 	ai->max_dirty_data_idx = 0;
 
-retry:
 	ret = read_object(ai, (char *)&ai->inode, vid_to_vdi_oid(vid),
 			  0, SD_INODE_SIZE, 0, &need_reload);
 	if (ret)
